	program micchi
C
C Calculate chi2 values between results from runs of the microtrub sphere code
C and an input data file.
C
c  History:
c    lgm  17jul92  Started based on radsph
c    lgm  01jun94  added logclose statement to end of main
c    lgm  06aug97   updated log writes to be compatable with next miriad subs
c
c------ Inline doc (retrieved with doc to a .doc file) ----------------c
c
c= micchi - Calculates chi2 bewteen obs and models from populations solution
c: models
c& lgm
c+
c  Calculates chi2 between emergent line intensity for a spherical gas 
C  distribution based on level population solutions from micsph and
C  observational data set input in data file.
c@ pops
c   MIRIAD image containing level population solutions output by
c   a program such as micsph.
c   NO default.
c@ cell
c   Spacing for calculation in arcseconds. The cell should
c   be chosen to be much smaller than the beam used of the observations.
c   A factor of 10 to 20 smaller than the beam diameter is best.
c   Default = 1.0
c@ dist
c   Distance to model cloud in pc. This is required to set the proper
c   scale in physical units.
c   Default = 150.
c@ data
c   Input data file containing positions, line freqs, tr, sigma for
c   each data point
c@ log
c   Name of output log file, if desired. Default is standard output.
c
c--------------------------------------------------------------------
c
        INCLUDE 'rad_size.h'
        INCLUDE 'radiat.h'
        INCLUDE 'mic_radtr.h'
	INCLUDE 'mic_radsup.h'
	INCLUDE 'mic_tau.h'
C
	INTEGER MAXSTEPS,MAXDATA
	PARAMETER (MAXDATA = 100)
	PARAMETER (MAXSTEPS = 512)
	CHARACTER popname*50,molcul*10
	CHARACTER logfile*50,line*100,words(20)*25
	CHARACTER datafile*50
	CHARACTER isotop*10,version*25
	INTEGER i,j,k,iprint,nsize(3),popin,nwords,inam,imol
	INTEGER ir,nsteps,ndata,rplane(MAXDATA)
	INTEGER fplane(MAXDATA),it,ib,nresol
	REAL rt,rb,bm,tr
	REAL csize,dist,row(MAXMOD),delrad
	REAL outfreq(MAXTRAN),chisqr,modval(MAXDATA)
	REAL newval,rpos,outres(MAXTRAN),beam(MAXTRAN,MAXSTEPS)
	REAL trpix(MAXTRAN,MAXSTEPS),sigma,x2
	REAL pos(MAXDATA),datfreq(MAXDATA),data(MAXDATA)
	REAL daterr(MAXDATA),resol(MAXDATA),epsln
	DOUBLE PRECISION sum,sumb
	LOGICAL eof,more,found
C
        INCLUDE 'radinit.h'
	DATA version / ' Version Aug 06, 1997 '/
C
	eof   = .false.
	more  = .true.
	CALL output('MICCHI:'//version)
	iprint = 1
C
C  Get inputs
C
	CALL KEYINI
	CALL KEYA('pops',popname,' ')
	if(popname .eq. ' ') 
     1      call bug('f','File with population must be specified')
	CALL KEYR('cell',csize,2.0)
	CALL KEYR('dist',dist,150.0)
	CALL KEYA('data',datafile,'  ')
	if(datafile(1:2) .eq. '  ') 
     1		call bug('f','Name of data file required')	
	CALL KEYA('log',logfile,' ')
	CALL KEYFIN
C
C   Open log file of specified
C
        call logopen(logfile,' ')
C
C  Open and read input model file -- expect output file from micsph
C	
	call xyopen(popin,popname,'old',3,nsize)
	do k=1,nsize(3)
           call xysetpl(popin,1,k)
           do j=1,nsize(2)
	      call xyread(popin,j,row)
              do i=1,nsize(1)
                 popn(j,i) = row(i)
              enddo
           enddo
	enddo
	nlev = nsize(2)
	call rdhda(popin,'object',molcul,' ')
	if(molcul .eq. ' ') 
     1      call bug('f','Molecule name not in model file')
C
C  Read in model info from the history file
C
        hole = .false.
	call hisopen(popin,'read')
	do while (.not.eof)
	   call hisread(popin,line,eof)
	   call sopchr(line,words,nwords)
	   if(words(2)(1:5) .eq. 'MODEL') then
	      if(words(3)(1:4) .eq. 'NMOD') read(words(4),*) npts
	      if(words(3)(1:3) .eq. 'ISO') isotop = words(4)
	      if(words(3)(1:3) .eq. 'SHE') then
	 	 do i=1,npts
		    call hisread(popin,line,eof)
		    call sopchr(line,words,nwords)
		    read(words(3),*) rpt(i)
		    read(words(4),*) rnh2(i)
		    read(words(5),*) rnx(i)
		    read(words(6),*) rtk(i)
 		    rpt(i)  = rpt(i)/au
		    rnh2(i) = alog(rnh2(i))
		    rnx(i)  = alog(rnx(i))
	 	 enddo
		 rmax = rpt(npts)
	      endif
	      if(words(3)(1:4) .eq. 'HOLE') then
		 if(words(4)(1:1) .eq. 'T') then
		    hole = .true.
		 else
		    hole = .false.
		 endif
	      endif
              if(words(3) .eq. 'DVEL') read(words(4),*) dopv
              if(words(3) .eq. 'TBKG') read(words(4),*) tbk 
	   endif
	enddo
	call hisclose(popin)
C
C   set-up interpolation arrays
C
        epsln = 1.0e-6
	call terp(popn,0,rpt(npts))
	call splnc(npts,rpt,rnh2,epsln,snh2)
	call splnc(npts,rpt,rnx,epsln,sxm)
C
C  Calculate molecular structure stuff
C
	call molread(molcul,inam,imol)
	if(inam .eq. 0) call bug('f','Molecule not found in list')
	call isoid(isotop,imol)
	call coll1(iprint,isotop,imol)
C
C   Open data file and read in data
C
        open(unit=22,file=datafile,status='old')
        do i=1,MAXDATA
           read(22,*,end=105) pos(i),datfreq(i),data(i),
     1                  daterr(i),resol(i)
           ndata = i
        enddo
  105   continue
C
C   Identify proper transition(s)
C
	do k=1,ndata
	   do i=1,nlev
	   do j=1,nlev
	      if(k .eq. 1) wrtrans(i,j) = .false.
	      if(abs(freq(i,j)-datfreq(k)) .lt. 0.1 
     1				.and. A(i,j) .gt. 0.0) then
         	 if(.not.wrtrans(i,j))then
		    wrtrans(i,j) = .true.
		    write(line,'(''    Found line freq: '',f10.3)')
     1				freq(i,j)
		    call LogWrite(line,more)
		 endif
	      endif
	   enddo
	   enddo
	enddo
C
C    Calculate line temperatures at each radius in steps of 1/2 of the
C    cell size desired.
C
	delrad  = csize*dist
	nsteps  = (rmax/delrad) + 55.
	if(nsteps .gt. MAXSTEPS) nsteps = MAXSTEPS
	do ir=1,nsteps
	   radius = delrad * (ir-1)
	   call trpeak(molcul,outfreq)
           do i=1,ifre
              trpix(i,ir) = trad(i)
           enddo
	enddo
C
C   Find out how many unique resolutions there are
C
	outres(1) = resol(1)
	nresol = 1
	do i=2,ndata
	   found=.false.
	   do j=1,nresol
	      if(resol(i) .eq. outres(j)) found=.true.
	   enddo
	   if(.not.found) then
	      nresol = nresol+1
	      outres(nresol) = resol(i)
	   endif
	enddo
C
C   Identify each data position with its proper resolution plane
C
	do i=1,ndata
	   do j=1,nresol
	      if(abs(outres(j)-resol(i)) .lt. 0.01) 
     1	         rplane(i) = j
	   enddo
	enddo

C
C  Identify each data position with its proper freq plane
C
	do i=1,ndata
	   do j=1,ifre
	     if(abs(outfreq(j)-datfreq(i)) .lt. 0.05) 
     1		fplane(i) = j
	   enddo
	enddo
C
C   Calculate a radial beam profile for each resolution
C
	do i=1,nresol
	   sigma = outres(i)/2.35482
	   do j=1,nsteps
	      x2 = ((j-1)*csize*(j-1)*csize)/(2.0*sigma*sigma)
	      if(x2 .lt. 25.0) then
	         beam(i,j) = exp(-x2)
	      else
		 beam(i,j) = 0.0
	      endif
	   enddo
	enddo
C
C    Calculate brightness temp for each data point in the proper beam
C
	do i=1,ndata
	   rpos = pos(i)/csize
	   sum  = 0.0d0
	   sumb = 0.0d0
	   do j=-nsteps,nsteps
	   do k=-nsteps,nsteps
	      rb = ((j*j + k*k)**0.5) + 1.001
	      rt = (((j-rpos)*(j-rpos) + k*k)**0.5) + 1.001
	      ib = rb
	      it = rt
	      if(ib .lt. nsteps .and. it .lt. nsteps) then
		 tr = trpix(fplane(i),it) + (trpix(fplane(i),it+1)
     1				-trpix(fplane(i),it))*(rt-it)
	         bm = beam(rplane(i),ib) + (beam(rplane(i),ib+1)
     1				-beam(rplane(i),ib))*(rb-ib)
		 sum =  sum + tr*bm
		 sumb = sumb + bm
	      endif
	   enddo
	   enddo
	   modval(i) = sum/sumb
	enddo
c
c  calculate chi-square
c
        chisqr = 0.0
        do i=1,ndata
           newval = (data(i)-modval(i))*(data(i)-modval(i))/
     1                  (daterr(i)*daterr(i))
           chisqr = chisqr + newval
        enddo
c
c   write out results
c
        call LogWrite('   Results of Chi-square fit:',more)
        call LogWrite('  ',more)
        do i=1,ndata
           write(line,'(''  Position: '',f5.1,'' Freq: '',f7.2,
     0        '' Obs. Tr: '',f6.2,
     1        '' Model Tr: '',f6.2,'' sigma: '',f6.3)')
     2         pos(i),datfreq(i),data(i),modval(i),daterr(i)
           call Logwrite(line,more)
        enddo
        if(ndata .gt. 1) then
	  write(line,'(''   Reduced Chi-sqr: '',f8.2)') chisqr/(ndata-1)
        else
	  write(line,'(''   Chi-sqr: '',f8.2)') chisqr
        endif
        call Logwrite(line,more)

        call logclose
	stop
	end
C****************************************************************
C**************** END OF MAIN ***********************************
C****************************************************************
      SUBROUTINE TRPEAK(MOLCUL,outfreq)
C
C   CALCULATES THE RADIAL OUTGOING EMISSION FROM THE SURFACE
C   FOR EACH OF THE MODEL TRANSITIONS AND EXPRESSES THEM IN
C   TR.  
C
      INCLUDE 'rad_size.h'
      INCLUDE 'radiat.h'
      INCLUDE 'mic_radsup.h'
      INCLUDE 'mic_tau.h'
C
      INTEGER I,J
      REAL T0,T1,DTU,TAUTOT,TBB,TR,outfreq(MAXTRAN)
      REAL tmax,tstp,sum,dtau,sout,gauss,epsln
      CHARACTER MOLCUL*10,LINE*80
      logical more
      EXTERNAL DTAU,SOUT,GAUSS
      data more / .true. /
C
c      call LogWrite(' ',more)
c      call LogWrite(' ',more)
c      WRITE(line,2000) MOLCUL,RADIUS*AU
c      call LogWrite(line,more)
c      call LogWrite(' ',more)
c      WRITE(line,2005)
c      call LogWrite(line,more)
      if(radius .gt. rmax) then
         do i=1,MAXTRAN
            trad(i) = 0.0 
         enddo 
c	 call LogWrite(' Beyond max radius of model; Emission zero!',
c    1				more)
         return 
      endif 
C
      rmu    = 0.0
      tmax   = ((rmax*rmax)-(radius*radius))**0.5
      if(tmax .le. 0.0) then
	 do i=1,MAXTRAN
	    trad(i) = 0.0
	 enddo
	 return
      endif
      tstp   = tmax/(maxt-1)
      ifre   = 0
c
c  loop over all possible transitions
c
      DO 100 JUP=2,NLEV
      DO 100 JLO=1,NLEV
         IF(A(JUP,JLO).LE.0.0) GO TO 100
	 IF(.not. wrtrans(jup,jlo)) go to 100
c
c   calculate tau versus t-distance 
c
         SUM=0.0
         AT(1)=0.0
         ATAU(1)=0.0
         DO 50 I=2,MAXT
            T0=TSTP*FLOAT(I-2)
            T1=TSTP*FLOAT(I-1)
            DTU=GAUSS(T0,T1,6,DTAU)
            SUM=SUM+DTU
            AT(I)=T1
   50    ATAU(I)=SUM
         TAUTOT=2.0*SUM
         epsln = 1.0e-6
         CALL SPLNC(MAXT,AT,ATAU,epsln,AA)
c
c divide integral for emission into MAXT part and sum them
c
	 sum = 0.0
         DO I=1,MAXT-1
	    t0 = -tmax + 2.0*(i-1)*tstp
	    t1 = t0 + 2.0*tstp
	    sum = sum + gauss(t0,t1,15,sout)
	 enddo
         TBB  = (EXP(-TAUTOT)-1.0)*BBK(JUP,JLO) + sum
         I    = JUP - 1
         J    = JLO - 1
         TR   = 0.5E-18*((AC/FREQ(JUP,JLO))**2)*TBB/AK
	 ifre = ifre + 1
	 trad(ifre) = tr
	 outfreq(ifre) = freq(jup,jlo)
c         WRITE(line,2010) I,J,TAUTOT,TR,FREQ(JUP,JLO)
c         call LogWrite(line,more)
  100 CONTINUE
c
 2000 FORMAT(7X,A5,29H LINE RADIATION FOR RADIUS = ,1pe11.3,
     1			3H cm)
 2005 FORMAT(5X,2HJU,2X,2HJL,8X,3HTAU,10X,8HRAD TEMP,9X,4HFREQ)
 2010 FORMAT(5X,I2,2X,I2,5X,1PE10.3,5X,E10.3,5X,0PF9.3)
      RETURN
      END
C******************************************************************
C************************* END OF TROUT ***************************
C******************************************************************
      FUNCTION SOUT(XLEN)
C
C   CALCULATES THE EMISSION FROM A LAYER AS SEEN FROM THE
C   SURFACE.
C
C INPUT: XLEN   -
C        SOUR   - SOURCE FUNCTION FOR LAYER
C
      REAL XLEN,AXL,SDT,SOUT,TAU,SOUDTAU
      REAL XT(5),YT(5)
      EXTERNAL SPLN,SOUDTAU
C
      INCLUDE 'mic_radsup.h'
      INCLUDE 'mic_tau.h'
C
      AXL   = ABS(XLEN)
      XT(1) = AXL
      CALL SPLN(MAXT,1,AT,ATAU,XT,YT,AA)
      IF(XLEN.LE.0.0) TAU = ATAU(MAXT)+YT(1)
      IF(XLEN.GT.0.0) TAU = ATAU(MAXT)-YT(1)
      SDT   = SOUDTAU(AXL)
      SOUT = SDT*EXP(-TAU)
      RETURN
      END
C****************************************************************
C***************************** END OF SOUT **********************
C****************************************************************
      FUNCTION DTAU(TDIS)
C
C   CALCULATES THE INCREMENTAL TAU AT THE DISTANCE TD ALONG AT RAY
C   AT ANGLE RMU ORIGINATING FROM RADIUS
C
C   RVAL IS THE TRUE RADIUS CORRESPONDING TO THE POINT OF DISTANCE
C   TD ALONG THE RAY UNDER CONSIDERATION
C
      REAL CTAU,RIPT,TDIS,RVAL,DH2,XMOL,DTAU
      REAL XT(5),YT(5)
C
      INCLUDE 'rad_size.h'
      INCLUDE 'radiat.h'
      INCLUDE 'mic_radtr.h'
      INCLUDE 'mic_radsup.h'
      INCLUDE 'mic_tau.h'
C
C
C      CTAU=1.6651E-9*AH*AC*AU/(4.0*PI*RPI*1.0E5*DOPV)
c
      CTAU = 2.22181e-9/dopv
      RIPT = RADIUS 
      RVAL = SQRT(RIPT*RIPT+TDIS*TDIS+2.0*RIPT*TDIS*RMU)
      XT(1)=RVAL
      IF(RVAL .LE. RPT(NPTS)) THEN
         CALL SPLN(NPTS,1,RPT,RNH2,XT,YT,SNH2)
         DH2  = EXP(YT(1))
         CALL SPLN(NPTS,1,RPT,RNX,XT,YT,SXM)
         XMOL = EXP(YT(1))
      ELSE
         DH2  = 0.0
	 XMOL = 0.0
      ENDIF
      if(rval .lt. rpt(1) .and. hole) then
	dh2  = 0.0
	xmol = 0.0
      endif
      CALL TERP(POPN,1,RVAL)
      DTAU = DH2 * XMOL * (XLO*BA(JLO,JUP) - XUP*BE(JUP,JLO))
     1			*exp(-2.76125*delv*delv/(dopv*dopv))
      if(abs(dtau) .gt. 1.0e-22) then
         dtau = ctau*dtau
      else
         dtau = 0.0
      endif
      RETURN
      END
C*******************************************************************
C*************************** END OF DTAU ***************************
C*******************************************************************
      FUNCTION SOUDTAU(TDIS)
C
C   CALCULATES THE INCREMENTAL SOURCE FUNCTION * dTAU AT THE DISTANCE TD ALONG 
C   A RAY AT ANGLE RMU ORIGINATING FROM RADIUS
C
C   RVAL IS THE TRUE RADIUS CORRESPONDING TO THE POINT OF DISTANCE
C   TD ALONG THE RAY UNDER CONSIDERATION
C
      REAL CTAU,RIPT,TDIS,RVAL,DH2,XMOL,DTAU
      REAL XT(5),YT(5),SOU,SOUDTAU
C
      INCLUDE 'rad_size.h'
      INCLUDE 'radiat.h'
      INCLUDE 'mic_radtr.h'
      INCLUDE 'mic_radsup.h'
      INCLUDE 'mic_tau.h'
C
C
C     CTAU=1.6651E-9*AH*AC*AU/(4.0*PI*RPI*1.0E5*DOPV)
c
      CTAU = 2.22181e-9/dopv
      RIPT = RADIUS 
      RVAL = SQRT(RIPT*RIPT+TDIS*TDIS+2.0*RIPT*TDIS*RMU)
      XT(1)=RVAL
      IF(RVAL .LE. RPT(NPTS)) THEN
         CALL SPLN(NPTS,1,RPT,RNH2,XT,YT,SNH2)
         DH2  = EXP(YT(1))
      ELSE
         DH2=0.0
      ENDIF
      CALL SPLN(NPTS,1,RPT,RNX,XT,YT,SXM)
      XMOL = EXP(YT(1))
      if(rval .lt. rpt(1) .and. hole) then
         dh2  = 0.0
         xmol = 0.0
      endif
      CALL TERP(POPN,1,RVAL)
      DTAU = DH2 * XMOL * BA(JLO,JUP)/DEGEN(JUP)
      SOU  = 2.0E+18*AH*(FREQ(JUP,JLO)**3)/(AC*AC) *
     1                  (XUP * DEGEN(JLO))
      SOUDTAU = SOU*DTAU
      if(abs(soudtau) .gt. 1.0e-22) then
         soudtau = ctau*soudtau
      else
         soudtau = 0.0
      endif
      RETURN
      END
C*******************************************************************
C************************** END OF SOUDTAU *************************
C*******************************************************************
      SUBROUTINE TRVEL(MOLCUL,nvsteps,vstep)
C
C   CALCULATES THE RADIAL OUTGOING EMISSION FROM THE SURFACE
C   FOR A RANGE OF VELOCITIES OF A TRANSITION AND EXPRESSES THEM IN
C   TR.
C
      INCLUDE 'rad_size.h'
      INCLUDE 'radiat.h'
      INCLUDE 'mic_radsup.h'
      INCLUDE 'mic_tau.h'
C
      INTEGER I,II,nvsteps
      REAL T0,T1,DTU,TAUTOT,TBB,TR,epsln
      REAL tmax,tstp,sum,dtau,sout,gauss,vstep
      CHARACTER MOLCUL*10,LINE*80
      logical more
      EXTERNAL DTAU,SOUT,GAUSS
      data more / .true. /
C       
      call LogWrite(' ',more)
      WRITE(line,2000) MOLCUL,RADIUS
      call LogWrite(line,more)
      call LogWrite(' ',more)
      if(radius .gt. rmax) then
         do i=1,MAXTRAN
            trad(i) = 0.0
         enddo
         call LogWrite(' Beyond max radius of model; Emission zero!',
     1                          more)
         return
      endif
C
      rmu    = 0.0
      tmax   = ((rmax*rmax)-(radius*radius))**0.5
      if(tmax .le. 0.0) then
         do i=1,MAXTRAN
            trad(i) = 0.0
         enddo
         return
      endif
      tstp    = tmax/(maxt-1)
      ifre    = nvsteps
      do ii=1,nvsteps
	 delv    = vstep*ifix(ii - (nvsteps/2 + 0.6))
         SUM     = 0.0
         AT(1)   = 0.0
         ATAU(1) = 0.0
         DO I=2,MAXT
            T0      = TSTP*FLOAT(I-2)
            T1      = TSTP*FLOAT(I-1)
            DTU     = GAUSS(T0,T1,10,DTAU)
            SUM     = SUM+DTU
            AT(I)   = T1
            ATAU(I) = SUM
         ENDDO 
         TAUTOT=2.0*SUM
         epsln = 1.0e-6
         CALL SPLNC(MAXT,AT,ATAU,epsln,AA)
c
c divide integral for emission into MAXT part and sum them
c
         sum = 0.0
         DO I=1,MAXT-1
            t0 = -tmax + 2.0*(i-1)*tstp
            t1 = t0 + 2.0*tstp
            sum = sum + gauss(t0,t1,15,sout)
         enddo
c
         TBB  = (EXP(-TAUTOT)-1.0)*BBK(JUP,JLO) + sum
         TR   = 0.5E-18*(AC/FREQ(JUP,JLO))**2*TBB/AK
         trad(ii) = tr
      enddo
 2000 FORMAT(7X,A5,29H LINE RADIATION FOR RADIUS = ,1pe11.3,
     1                  3H cm)
      RETURN
      END
C******************************************************************
C************************* END OF TRVEL ***************************
C******************************************************************

