	program radsph
C
C  Displays the results from runs of the microtrub sphere code.
C  A data file containing model information as output from micsph
C  must be specified.
C
c  History:
c    lgm  pre-his  program begun
c    lgm  30jun92  working version
c    lgm  16jul92  Modified to work in distance units of au
c    lgm  17jul92  worrying about accuracy of integrations for Tr when
c                    strong gradients are present. Upped order of gaussian
c                    in tau and sout integration. Changed to calculate
c                    sout over MAXT intervals.
c    lgm 01jun94   added logclose at end of main
c    lgm  06aug97   updated log writes to be compatable with next miriad subs
c
c------ Inline doc (retrieved with doc to a .doc file) ----------------c
c
c= radsph - Caculates emergent lines for a spherical gas distribution
c: models
c& lgm
c+
c  Calculates emergent line intensity for a spherical gas distribution
c  based on level population solutions supplied in the form of an
c  image created by micsph.
c@ pops
c   MIRIAD image containing level population solutions output by
c   a program such as micsph.
c   NO default.
c@ cell
c   Pixel size for output image in arcseconds. The pixel size should
c   be chosen to be much smaller than the beam used of the observations.
c   The program smooth can be used to derive maps for comparison with
c   data.
c   Default = 2.0
c@ dist
c   Distance to model cloud in pc. This is required to set the proper
c   scale in physical units.
c   Default = 150.
c@ options
c   User selected option to specify what type of output cube you want.
c   Allowed options are: peak -- cube contains peak emission map with
c                                3'rd axis as transition
c                        vel  -- cube contains emission map with 3`rd
c                                axis as velocity. NOTE: When this option
c                                is chosen you MUST also velocity step between 
c                                planes (in km/sec). For example, 
c                                options=vel,0.2
c   Default is peak.
c@ trans
c   Frequency(s)  of transitions to write to output cube. Up to 10 freqs
c   maybe specified for options=peak; only one freq can be specified for
c   options=peak. Frequencies are in GHz.
c   Default: none
c@ print
c   Parameter which control level of printed output.
c      0 = minimum output which is useful when running grids
c      1 = normal output level
c      2 = detailed output level - used for debugging and error checking
c@ out
c   Name of output MIRIAD image. No default.
c@ log
c   Name of output log file, if desired. Default is standard output.
c
c--------------------------------------------------------------------
c
        INCLUDE 'rad_size.h'
        INCLUDE 'radiat.h'
        INCLUDE 'radinit.h'
        INCLUDE 'mic_radtr.h'
	INCLUDE 'mic_radsup.h'
	INCLUDE 'mic_tau.h'
C
	INTEGER MAXSTEPS
	PARAMETER (MAXSTEPS = 512)
	CHARACTER popname*50,molcul*10,option*12,outname*50
	CHARACTER logfile*50,line*90,words(20)*25
	CHARACTER isotop*10,ctyp(3)*8,version*25
	INTEGER i,j,k,iprint,nsize(3),popin,nwords,inam,imol
	INTEGER ir,nsteps,out,nvsteps,ntrans
	REAL csize,dist,row(MAXMOD),pixsiz,delrad
	REAL tranfr(MAXTRAN),outfreq(MAXTRAN),vstep
	REAL trpix(MAXTRAN,MAXSTEPS),rpix(MAXSTEPS)
	REAL crval(3),crdel(3),epsln
	LOGICAL eof,more,keyprsnt
	EXTERNAL keyprsnt
C
	DATA version / ' Version Aug 06, 1997   '/

C
	eof   = .false.
	more  = .true.
	vstep = -0.1
	CALL output('RADSPH:'//version)
C
C  Get inputs
C
	CALL KEYINI
	CALL KEYA('pops',popname,' ')
	if(popname .eq. ' ') 
     1      call bug('f','File with population must be specified')
	CALL KEYR('cell',csize,2.0)
	CALL KEYR('dist',dist,150.0)
        CALL KEYA('options',option,' ')
	if(option(1:3) .eq. 'vel') then
	   CALL KEYR('options',vstep,-0.1)
	   CALL output(' Creating velocity data cube')
	else
	   option = 'peak'
	   CALL output(' Creating transition data cube')
	endif
	CALL KEYR('trans',tranfr(1),-1.0)
	if(tranfr(1) .le. 0.0) then
	   CALL bug('f','One or more freqs must be specified')
	endif
	ntrans = 1
	dowhile (keyprsnt('trans'))
	   ntrans = ntrans + 1
	   if(ntrans .le. MAXTRAN)
     1           CALL KEYR('trans',tranfr(ntrans),-1.0)
	enddo
	if(option(1:3) .eq. 'vel' .and. ntrans .gt. 1) then
	   CALL bug('f','Only one transition possible with option=vel')
	endif
	CALL KEYA('out',outname,' ')
	if(outname .eq. ' ')
     1      call bug('f','Output image must be specified')
C   
C   Input print level specification
C
        CALL KEYI('print',iprint,1)
C   
	CALL KEYA('log',logfile,' ')
	CALL KEYFIN
C
C   Open log file of specified
C
        call logopen(logfile,' ')
C
C  Open and read input model file -- expect output file from micsph
C	
	call xyopen(popin,popname,'old',3,nsize)
	do k=1,nsize(3)
           call xysetpl(popin,1,k)
           do j=1,nsize(2)
	      call xyread(popin,j,row)
              do i=1,nsize(1)
                 popn(j,i) = row(i)
              enddo
           enddo
	enddo
	nlev = nsize(2)
	call rdhda(popin,'object',molcul,' ')
	if(molcul .eq. ' ') 
     1      call bug('f','Molecule name not in model file')
C
C  Read in model info from the history file
C
        hole = .false.
	call hisopen(popin,'read')
	do while (.not.eof)
	   call hisread(popin,line,eof)
	   call sopchr(line,words,nwords)
	   if(words(2)(1:5) .eq. 'MODEL') then
	      if(words(3)(1:4) .eq. 'NMOD') read(words(4),*) npts
	      if(words(3)(1:3) .eq. 'ISO') isotop = words(4)
	      if(words(3)(1:3) .eq. 'SHE') then
	 	 do i=1,npts
		    call hisread(popin,line,eof)
		    call sopchr(line,words,nwords)
		    read(words(3),*) rpt(i)
		    read(words(4),*) rnh2(i)
		    read(words(5),*) rnx(i)
		    read(words(6),*) rtk(i)
 		    rpt(i)  = rpt(i)/au
		    rnh2(i) = alog(rnh2(i))
		    rnx(i)  = alog(rnx(i))
	 	 enddo
		 rmax = rpt(npts)
	      endif
	      if(words(3)(1:4) .eq. 'HOLE') then
		 if(words(4)(1:1) .eq. 'T') then
		    hole = .true.
		 else
		    hole = .false.
		 endif
	      endif
              if(words(3) .eq. 'DVEL') read(words(4),*) dopv
              if(words(3) .eq. 'TBKG') read(words(4),*) tbk 
	   endif
	enddo
	call hisclose(popin)
C
C   set-up interpolation arrays
C
        epsln = 1.0e-6
	call terp(popn,0,rpt(npts))
	call splnc(npts,rpt,rnh2,epsln,snh2)
	call splnc(npts,rpt,rnx,epsln,sxm)
C
C  Calculate molecular structure stuff
C
	call molread(molcul,inam,imol)
	if(inam .eq. 0) call bug('f','Molecule not found in list')
	call isoid(isotop,imol)
	call coll1(iprint,isotop,imol)
C
C   Identify proper transition(s)
C
	if(option(1:3) .eq. 'vel') then
	   jup = -1
	   do i=1,nlev
	   do j=1,nlev
	      if(abs(freq(i,j)-tranfr(1)) .lt. 0.1) then
		 jup = i
		 jlo = j
	      endif
	   enddo
	   enddo
	   if(jup .le. 0) call bug('f',' Transition freq not found')
	endif
	if(option(1:3) .eq. 'pea') then
	  ifre = 0
	  do k=1,ntrans
	    do i=1,nlev
	    do j=1,nlev
	      if(k .eq. 1) wrtrans(i,j) = .false.
	      if(abs(freq(i,j)-tranfr(k)) .lt. 0.1 
     1	               .and. A(i,j) .gt. 0) then
                wrtrans(i,j) = .true.
		ifre = ifre+1
		outfreq(ifre) = freq(i,j)
		write(line,'('' Found line freq: '',f10.3)')
     1				freq(i,j)
		call LogWrite(line,more)
	      endif
	    enddo
	    enddo
	  enddo
	endif
C
C    Calculate line temperatures at each radius in steps of 1/2 of the
C    cell size desired.
C
	pixsiz  = csize*dist
	delrad  = pixsiz/2.0
	nsteps  = (rmax/delrad) + 2.1
	nvsteps = 2.5*dopv/vstep + 2.1
	if(nsteps .gt. MAXSTEPS) nsteps = MAXSTEPS
	do ir=1,nsteps
	   radius = delrad * (ir-1)
	   if(option(1:3) .eq. 'pea') then
	        delv = 0.0
		call trpeak(molcul)
	   else
		call trvel(molcul,nvsteps,vstep)
	   endif
	   rpix(ir) = radius/pixsiz
	   do i=1,ifre
	      trpix(i,ir) = trad(i)
	   enddo
	enddo
C
C   Make output image
C
	nsize(1) = 4.0*(rmax/pixsiz + 10.0)
	nsize(2) = nsize(1)
	nsize(3) = ifre
	ctyp(1)  = 'RA---SIN'
	ctyp(2)  = 'DEC--SIN'
	crval(1) = 0.0
        crval(2) = 0.0
 	crdel(1) = -csize/206265.
	crdel(2) = csize/206265.
	if(option(1:3) .eq. 'pea') then
	   ctyp(3)  = 'LINE'
	   crval(3) = 0.0
	   crdel(3) = 1.0
	else
	   ctyp(3)  = 'FELO-LSR'
	   crval(3) = -vstep*(nvsteps/2)
	   crdel(3) = vstep
	endif
	call xyopen(out,outname,'new',3,nsize)
        call wrhead(out,molcul,nsize,ctyp,crval,crdel)
	call makimag(out,csize,nsize,MAXSTEPS,MAXTRAN,
     1				rpix,trpix,nsteps)
	call hisopen(out,'write')
	call hisappn(out,popname,.false.)
	call hiswrite(out,'RADSPH:'//version)
	call hisinput(out,'radsph')
	if(hole) call hiswrite(out,'RADSPH: Central hole present')
	call hiswrite(out,'RADSPH: Transitions written to planes')
        do i=1,ifre
	   write(line,'(''RADSPH: Plane # '',i2,'' Freq: '',f10.3,
     1			'' GHz'')') i,outfreq(i)
	   call hiswrite(out,line)
	enddo
	call hisclose(out)
	call xyclose(out)
        call logclose
C
	stop
	end
C****************************************************************
C**************** END OF MAIN ***********************************
C****************************************************************
	SUBROUTINE makimag(out,csize,nsize,MAXSTEPS,MAXTRAN,
     1                  rpix,trpix,nsteps)
C
	INTEGER out,nsize(3),MAXSTEPS,nsteps
        INTEGER MAXTRAN
	REAL csize,rpix(1),trpix(MAXTRAN,MAXSTEPS)
	REAL row(1024),relr
	INTEGER i,j,k,ii,cenx,ceny,pix
C
	cenx = nsize(1)/2
	ceny = nsize(2)/2
	do k=1,nsize(3)
	   call xysetpl(out,1,k)
	   do j=1,nsize(2)
	      do i=1,nsize(1)
		 relr = ((i-cenx)*(i-cenx)+(j-ceny)*(j-ceny))**0.5
		 pix  = 0
		 do ii=1,nsteps-1
		    if(relr.ge.rpix(ii) .and. relr.lt.rpix(ii+1)) 
     1				pix = ii
		 enddo
	 	 if(pix .gt. 0) then
		    row(i) = trpix(k,pix) + 
     1			((relr-rpix(pix))/(rpix(pix+1)-rpix(pix)))*
     2			(trpix(k,pix+1)-trpix(k,pix))
		 else
		    row(i) = 0.0
		 endif
	      enddo
	      call xywrite(out,j,row)
	   enddo
	enddo
C
	return
	end
C
C**************************************************************
        subroutine wrhead(lout,molcul,nsize,ctyp,crval,crdel)
C
C  writes header for image cube
C
        integer lout,nsize(3)
        real crval(3),crdel(3)
        character molcul*10,ctyp(3)*8
C
        call wrhdi(lout,'naxis',3)
        call wrhdi(lout,'naxis1',nsize(1))
        call wrhdi(lout,'naxis2',nsize(2))
        call wrhdi(lout,'naxis3',nsize(3))
        call wrhda(lout,'ctype1',ctyp(1))
        call wrhda(lout,'ctype2',ctyp(2))
        call wrhda(lout,'ctype3',ctyp(3))
        call wrhdd(lout,'crpix1',dble(nsize(1)/2.0+1.0))
        call wrhdd(lout,'crpix2',dble(nsize(2)/2.0+1.0))
        call wrhdd(lout,'crpix3',1.0d0)
        call wrhdd(lout,'cdelt1',dble(crdel(1)))
        call wrhdd(lout,'cdelt2',dble(crdel(2)))
        call wrhdd(lout,'cdelt3',dble(crdel(3)))
        call wrhdd(lout,'crval1',dble(crval(1)))
        call wrhdd(lout,'crval2',dble(crval(2)))
        call wrhdd(lout,'crval3',dble(crval(3)))
        call wrhda(lout,'object',molcul)
        return
        end
C
C*****************************************************************
      SUBROUTINE TRPEAK(MOLCUL)
C
C   CALCULATES THE RADIAL OUTGOING EMISSION FROM THE SURFACE
C   FOR EACH OF THE MODEL TRANSITIONS AND EXPRESSES THEM IN
C   TR.  
C
      INCLUDE 'rad_size.h'
      INCLUDE 'radiat.h'
      INCLUDE 'mic_radsup.h'
      INCLUDE 'mic_tau.h'
C
      INTEGER I,J
      REAL T0,T1,DTU,TAUTOT,TBB,TR,epsln
      REAL tmax,tstp,sum,dtau,sout,gauss
      CHARACTER MOLCUL*10,LINE*80
      logical more
      EXTERNAL DTAU,SOUT,GAUSS
      data more / .true. /
C
      call LogWrite(' ',more)
      call LogWrite(' ',more)
      WRITE(line,2000) MOLCUL,RADIUS*AU
      call LogWrite(line,more)
      call LogWrite(' ',more)
      WRITE(line,2005)
      call LogWrite(line,more)
      if(radius .gt. rmax) then
         do i=1,MAXTRAN
            trad(i) = 0.0 
         enddo 
	 call LogWrite(' Beyond max radius of model; Emission zero!',
     1				more)
         return 
      endif 
C
      rmu    = 0.0
      tmax   = ((rmax*rmax)-(radius*radius))**0.5
      if(tmax .le. 0.0) then
	 do i=1,MAXTRAN
	    trad(i) = 0.0
	 enddo
	 return
      endif
      tstp   = tmax/(maxt-1)
      ifre   = 0
c
c  loop over all possible transitions
c
      DO 100 JUP=2,NLEV
      DO 100 JLO=1,NLEV
         IF(A(JUP,JLO).LE.0.0) GO TO 100
	 IF(.not. wrtrans(jup,jlo)) go to 100
c
c   calculate tau versus t-distance 
c
         SUM=0.0
         AT(1)=0.0
         ATAU(1)=0.0
         DO 50 I=2,MAXT
            T0=TSTP*FLOAT(I-2)
            T1=TSTP*FLOAT(I-1)
            DTU=GAUSS(T0,T1,6,DTAU)
            SUM=SUM+DTU
            AT(I)=T1
   50    ATAU(I)=SUM
         TAUTOT=2.0*SUM
         epsln = 1.0e-6
         CALL SPLNC(MAXT,AT,ATAU,epsln,AA)
c
c divide integral for emission into MAXT part and sum them
c
	 sum = 0.0
         DO I=1,MAXT-1
	    t0 = -tmax + 2.0*(i-1)*tstp
	    t1 = t0 + 2.0*tstp
	    sum = sum + gauss(t0,t1,15,sout)
	 enddo
         TBB  = (EXP(-TAUTOT)-1.0)*BBK(JUP,JLO) + sum
         I    = JUP - 1
         J    = JLO - 1
         TR   = 0.5E-18*((AC/FREQ(JUP,JLO))**2)*TBB/AK
	 ifre = ifre + 1
	 trad(ifre) = tr
         WRITE(line,2010) I,J,TAUTOT,TR,FREQ(JUP,JLO)
         call LogWrite(line,more)
  100 CONTINUE
c
 2000 FORMAT(7X,A5,29H LINE RADIATION FOR RADIUS = ,1pe11.3,
     1			3H cm)
 2005 FORMAT(5X,2HJU,2X,2HJL,8X,3HTAU,10X,8HRAD TEMP,9X,4HFREQ)
 2010 FORMAT(5X,I2,2X,I2,5X,1PE10.3,5X,E10.3,5X,0PF9.3)
      RETURN
      END
C******************************************************************
C************************* END OF TROUT ***************************
C******************************************************************
      FUNCTION SOUT(XLEN)
C
C   CALCULATES THE EMISSION FROM A LAYER AS SEEN FROM THE
C   SURFACE.
C
C INPUT: XLEN   -
C        SOUR   - SOURCE FUNCTION FOR LAYER
C
      REAL XLEN,AXL,SDT,SOUT,TAU,SOUDTAU
      REAL XT(5),YT(5)
      EXTERNAL SPLN,SOUDTAU
C
      INCLUDE 'mic_radsup.h'
      INCLUDE 'mic_tau.h'
C
      AXL   = ABS(XLEN)
      XT(1) = AXL
      CALL SPLN(MAXT,1,AT,ATAU,XT,YT,AA)
      IF(XLEN.LE.0.0) TAU = ATAU(MAXT)+YT(1)
      IF(XLEN.GT.0.0) TAU = ATAU(MAXT)-YT(1)
      SDT   = SOUDTAU(AXL)
      SOUT = SDT*EXP(-TAU)
      RETURN
      END
C****************************************************************
C***************************** END OF SOUT **********************
C****************************************************************
      FUNCTION DTAU(TDIS)
C
C   CALCULATES THE INCREMENTAL TAU AT THE DISTANCE TD ALONG AT RAY
C   AT ANGLE RMU ORIGINATING FROM RADIUS
C
C   RVAL IS THE TRUE RADIUS CORRESPONDING TO THE POINT OF DISTANCE
C   TD ALONG THE RAY UNDER CONSIDERATION
C
      REAL CTAU,RIPT,TDIS,RVAL,DH2,XMOL,DTAU
      REAL XT(5),YT(5)
C
      INCLUDE 'rad_size.h'
      INCLUDE 'radiat.h'
      INCLUDE 'mic_radtr.h'
      INCLUDE 'mic_radsup.h'
      INCLUDE 'mic_tau.h'
C
C
C      CTAU=1.6651E-9*AH*AC*AU/(4.0*PI*RPI*1.0E5*DOPV)
c
      CTAU = 2.22181e-9/dopv
      RIPT = RADIUS 
      RVAL = SQRT(RIPT*RIPT+TDIS*TDIS+2.0*RIPT*TDIS*RMU)
      XT(1)=RVAL
      IF(RVAL .LE. RPT(NPTS)) THEN
         CALL SPLN(NPTS,1,RPT,RNH2,XT,YT,SNH2)
         DH2  = EXP(YT(1))
         CALL SPLN(NPTS,1,RPT,RNX,XT,YT,SXM)
         XMOL = EXP(YT(1))
      ELSE
         DH2  = 0.0
	 XMOL = 0.0
      ENDIF
      if(rval .lt. rpt(1) .and. hole) then
	dh2  = 0.0
	xmol = 0.0
      endif
      CALL TERP(POPN,1,RVAL)
      DTAU = DH2 * XMOL * (XLO*BA(JLO,JUP) - XUP*BE(JUP,JLO))
     1			*exp(-2.76125*delv*delv/(dopv*dopv))
      if(abs(dtau) .gt. 1.0e-22) then
         dtau = ctau*dtau
      else
         dtau = 0.0
      endif
      RETURN
      END
C*******************************************************************
C*************************** END OF DTAU ***************************
C*******************************************************************
      FUNCTION SOUDTAU(TDIS)
C
C   CALCULATES THE INCREMENTAL SOURCE FUNCTION * dTAU AT THE DISTANCE TD ALONG 
C   A RAY AT ANGLE RMU ORIGINATING FROM RADIUS
C
C   RVAL IS THE TRUE RADIUS CORRESPONDING TO THE POINT OF DISTANCE
C   TD ALONG THE RAY UNDER CONSIDERATION
C
      REAL CTAU,RIPT,TDIS,RVAL,DH2,XMOL,DTAU
      REAL XT(5),YT(5),SOU,SOUDTAU
C
      INCLUDE 'rad_size.h'
      INCLUDE 'radiat.h'
      INCLUDE 'mic_radtr.h'
      INCLUDE 'mic_radsup.h'
      INCLUDE 'mic_tau.h'
C
C
C     CTAU=1.6651E-9*AH*AC*AU/(4.0*PI*RPI*1.0E5*DOPV)
c
      CTAU = 2.22181e-9/dopv
      RIPT = RADIUS 
      RVAL = SQRT(RIPT*RIPT+TDIS*TDIS+2.0*RIPT*TDIS*RMU)
      XT(1)=RVAL
      IF(RVAL .LE. RPT(NPTS)) THEN
         CALL SPLN(NPTS,1,RPT,RNH2,XT,YT,SNH2)
         DH2  = EXP(YT(1))
      ELSE
         DH2=0.0
      ENDIF
      CALL SPLN(NPTS,1,RPT,RNX,XT,YT,SXM)
      XMOL = EXP(YT(1))
      if(rval .lt. rpt(1) .and. hole) then
         dh2  = 0.0
         xmol = 0.0
      endif
      CALL TERP(POPN,1,RVAL)
      DTAU = DH2 * XMOL * BA(JLO,JUP)/DEGEN(JUP)
      SOU  = 2.0E+18*AH*(FREQ(JUP,JLO)**3)/(AC*AC) *
     1                  (XUP * DEGEN(JLO))
      SOUDTAU = SOU*DTAU
      if(abs(soudtau) .gt. 1.0e-22) then
         soudtau = ctau*soudtau
      else
         soudtau = 0.0
      endif
      RETURN
      END
C*******************************************************************
C************************** END OF SOUDTAU *************************
C*******************************************************************
      SUBROUTINE TRVEL(MOLCUL,nvsteps,vstep)
C
C   CALCULATES THE RADIAL OUTGOING EMISSION FROM THE SURFACE
C   FOR A RANGE OF VELOCITIES OF A TRANSITION AND EXPRESSES THEM IN
C   TR.
C
      INCLUDE 'rad_size.h'
      INCLUDE 'radiat.h'
      INCLUDE 'mic_radsup.h'
      INCLUDE 'mic_tau.h'
C
      INTEGER I,II,nvsteps
      REAL T0,T1,DTU,TAUTOT,TBB,TR,epsln
      REAL tmax,tstp,sum,dtau,sout,gauss,vstep
      CHARACTER MOLCUL*10,LINE*80
      logical more
      EXTERNAL DTAU,SOUT,GAUSS
      data more / .true. /
C       
      call LogWrite(' ',more)
      WRITE(line,2000) MOLCUL,RADIUS
      call LogWrite(line,more)
      call LogWrite(' ',more)
      if(radius .gt. rmax) then
         do i=1,MAXTRAN
            trad(i) = 0.0
         enddo
         call LogWrite(' Beyond max radius of model; Emission zero!',
     1                          more)
         return
      endif
C
      rmu    = 0.0
      tmax   = ((rmax*rmax)-(radius*radius))**0.5
      if(tmax .le. 0.0) then
         do i=1,MAXTRAN
            trad(i) = 0.0
         enddo
         return
      endif
      tstp    = tmax/(maxt-1)
      ifre    = nvsteps
      do ii=1,nvsteps
	 delv    = vstep*ifix(ii - (nvsteps/2 + 0.6))
         SUM     = 0.0
         AT(1)   = 0.0
         ATAU(1) = 0.0
         DO I=2,MAXT
            T0      = TSTP*FLOAT(I-2)
            T1      = TSTP*FLOAT(I-1)
            DTU     = GAUSS(T0,T1,10,DTAU)
            SUM     = SUM+DTU
            AT(I)   = T1
            ATAU(I) = SUM
         ENDDO 
         TAUTOT=2.0*SUM
         epsln = 1.0e-6
         CALL SPLNC(MAXT,AT,ATAU,epsln,AA)
c
c divide integral for emission into MAXT part and sum them
c
         sum = 0.0
         DO I=1,MAXT-1
            t0 = -tmax + 2.0*(i-1)*tstp
            t1 = t0 + 2.0*tstp
            sum = sum + gauss(t0,t1,15,sout)
         enddo
c
         TBB  = (EXP(-TAUTOT)-1.0)*BBK(JUP,JLO) + sum
         TR   = 0.5E-18*(AC/FREQ(JUP,JLO))**2*TBB/AK
         trad(ii) = tr
      enddo
 2000 FORMAT(7X,A5,29H LINE RADIATION FOR RADIUS = ,1pe11.3,
     1                  3H cm)
      RETURN
      END
C******************************************************************
C************************* END OF TRVEL ***************************
C******************************************************************

