        subroutine isoid(isotop,imol)
c---------------------------------------------------------------------
c    This subroutine allows isotopomers of the molecules with calculated
c    collision rates to be entered either as recognized "normal"
c    isotopes (see LINLIST, ASYLIST, or SYMLIST) or as user input 
c    isotopomers.  The constants for the "normal" isotopes come from 
c    Journal of Physical and Chemical Reference Data articles.  
c    Specifically...
c
c                     CO - vol. 3, p. 245
c                     CS - vol. 3, p. 252
c                    OCS - vol. 3, p. 224
c                    HCN - vol. 3, p. 236
c                   HC3N - vol. 7, p. 445
C   HCOP - new dipole moment from Haese and Woods 1979 (Chem. Phys. Lets.
C                  61, 396)
c                   SIO  - vol. 3, 
c                   H2CO - 
c                  CH3CN - vol. 9, p. 659
c--------------------------------------------------------------------
	INCLUDE 'rad_size.h'
	INCLUDE 'radiat.h'
C
        integer NISO,NASY,NSYM
        parameter (NISO = 27)
        parameter (NASY = 12)
        parameter (NSYM = 8)

        real bmol(NISO),demol(NISO),dylin(NISO),aasy(NASY),
     1      basy(NASY),casy(NASY),dyasy(NASY),bsym(NSYM),csym(NSYM),
     2      djsym(NSYM),djksym(NSYM),dksym(NSYM),dysym(NSYM)
        character*10 isotop,llist(NISO),alist(NASY),symlist(NSYM),
     1      oldiso
	character caps*26,alph*26
        data llist / 'CO','C17O','C18O','13CO','13C18O','CS','C33S',
     1      'C34S','13CS','OCS','OC34S','O13CS','18OCS','HCN',
     2      'H13CN','HC15N','DCN','D13CN','HC3N','HCC13CN','HC13CCN',
     3      'H13CCCN','DC3N','HCOP','SIO','29SIO','30SIO' /
        data bmol  / 57.63597,56.17998,54.89142,55.10102,52.35601,
     1      24.49557,24.29334,24.10355,23.12381,6.081492,5.932838,
     2      6.061925,5.704861,44.31597,43.17009,43.02769,36.20746,
     2      35.58762,4.549059,4.530198,4.529762,4.408442,
     3      4.221582,44.594,21.711954,21.439981,21.186702 /
        data demol / 183.567,174.388,166.462,167.745,151.405,40.237,
     1      39.574,38.774,35.849,1.3018,1.2691,1.299,1.1353,87.24,
     2      83.0,82.3,57.83,55.7,0.54695,0.5401,0.5438,0.5088,
     3      0.4530,80.0,29.87,29.113,28.44  /
        data dylin / 5*0.112, 4*1.96,4*0.715,5*2.98,5*3.724,4.48,
     1               3*3.0982 /
        data alist / 'OH2CO','PH2CO','OH213CO','PH213CO','OH2C18O',
     1      'PH2C18O','OH2CS','PH2CS','OH2C34S','PH2C34S','OH213CS',
     2      'PH213CS' /
        data aasy / 2*281.9704,2*281.9915,2*281.9330,6*291.6601 /
        data basy/ 2*38.83543,2*37.81050,2*36.90434,2*17.69956,
     1      2*17.38831,2*16.99870 /
        data casy / 2*34.00573,2*33.21738,2*32.51134,2*16.65306,
     1      2*16.37780,2*16.03165 /
        data dyasy / 6*2.33,6*1.65 /
        data symlist / 'OCH3CN','PCH3CN','O13CH3CN','P13CH3CN',
     1                 'OCH313CN','PCH313CN','OCH3C15N','PCH3C15N' /
        data bsym / 2*9.1989,2*8.9333,2*9.1943,2*8.9220 /
        data csym / 8*157.3 /
        data djsym / 2*3.8048,2*3.6740,2*3.8170,2*3.5788 /
        data djksym / 2*177.417,2*167.650,2*176.146,2*169.043 /
        data dksym / 2*2840.0,2*2850.0,2*2840.0,2*2850.0 /
        data dysym / 8*3.913 /
        data oldiso / 'old' /
        data alph / 'abcdefghijklmnopqrstuvwxyz'/
        data caps / 'ABCDEFGHIJKLMNOPQRSTUVWXYZ'/

        do i=1,10
           do j=1,26
              if(isotop(i:i) .eq. alph(j:j)) isotop(i:i) = caps(j:j)
           enddo
        enddo

        if(isotop.eq.oldiso) go to 405
        if(imol.eq.-1) go to 205
        if(imol.eq.-2) go to 206
   10 iis=0
        do 100 i=1,niso
          if(isotop.eq.llist(i)) iis=i
  100   continue
        if(iis.gt.0) then
           brot=bmol(iis)
           de=demol(iis)/1e+6
           dy=dylin(iis)
        else
           write(*,2000)
           write(*,2010) (llist(i),i=1,niso)
           write(*,2020)
           read(*,*) brot,de,dy
           if(dy.lt.1e-6) dy=1.0e18*dy
        end if
        oldiso=isotop
        return
  205 iis=0
        do 250 i=1,nasy
           if(isotop.eq.alist(i)) iis=i
  250 continue
        if(iis.gt.0) then
           a0=aasy(iis)
           b0=basy(iis)
           c0=casy(iis)
           dy=dyasy(iis)
        else
           write(*,2000)
           write(*,2010) (alist(i),i=1,nasy)
           write(*,2030)
           read(*,*) a0,b0,c0,dy
           if(dy.lt.1e-6) dy=1.0e18*dy
        end if
        oldiso=isotop
        return
  206 iis=0
        do 251 i=1,nsym
           if(isotop.eq.symlist(i)) iis=i
  251 continue
        if(iis.gt.0) then
           b0=bsym(iis)
           c0=csym(iis)
           dj=djsym(iis)/1.0e+6
           djk=djksym(iis)/1.0e+6
           dk=dksym(iis)/1.0e+6
           dy=dysym(iis)
        else
           write(*,2000)
           write(*,2010) (symlist(i),i=1,nsym)
           write(*,2040)
           read(*,*) b0,c0,dy
           if(dy.lt.1e-6) dy=1.0e18*dy
        end if
        oldiso=isotop
  405 return
 1000 format(a10)  
 2000 format(' Isotopomer does not appear on the known list ')
 2010 format(5x,5(a10,' , '))
 2020 format(' Enter B, DE, and dipole moment for your isotopomer ')
 2030 format(' Enter A, B, C, and dipole moment for your isotopomer ')
 2040 format(' Enter B, C, and dipole moment for your isotopomer ')
      end
c********************************************************************
c************************** END OF ISOID ****************************
c********************************************************************
