      subroutine radasy(iprint,isotop)
c--------------------------------------------------------------------
c    Computes radiative rates for slightly asymmetric tops.
c        A  - Einstein spon. emis. coef.
c        BE - Einstein stim emis coef
c        BA - Einstein abs coef
c        A0,B0,C0 are the rotation constants
c        S - line strength
c
c *** 07/09/90 ==> Changed the way the asymmetric top frequencies and
c                  emission coeficients are calculated.  Now reads in
c                  line strengths and energy levels and calculates
c                  everything from there.
c--------------------------------------------------------------------
      INCLUDE 'rad_size.h'
      INCLUDE 'radiat.h'
      integer j1(MAXLEV),km1(MAXLEV),kp1(MAXLEV),nelevs,nflavors
      integer i,ncat
      real sij(MAXLEV,MAXLEV),energy(MAXLEV,MAXLEV)
      character molist(6)*10,isotop*10,levfil(2)*11,line*80
      character catname*50,fullname*50
      real mua,mub,muc,mu,ss
      logical more
      data molist / 'OH2CO','OH213CO','OH2C18O','PH2CO','PH213CO',
     1              'PH2C18O' /
      data levfil / 'oh2co.level','ph2co.level' /
      data more / .true. /
      external fullname
c--------------------------------------------------------------------
c    If a new collision rates file is being opened, open and read in 
c    the level information and the absorption coefficients which 
c    designate the allowed transitions from the level information 
c    file.  The naming convention for this file is...
c
c                           molname.level
c
c    ...and is assigned to unit 13.
c--------------------------------------------------------------------
      ilev = 0
      do 10 i=1,8
         if(isotop .eq. molist(i)) ilev=i
 10   continue
      if((ilev.eq.1).or.(ilev.eq.2).or.(ilev.eq.3)) ilev=1
      if((ilev.eq.4).or.(ilev.eq.5).or.(ilev.eq.6)) ilev=2
      if(ilev.eq.0) then
         write(line,2001) isotop
         call bug('f',line)
      else
         catname = fullname('$RADCAT/') 
         do 11 i=1,50 
            if(catname(i:i) .ne. ' ') ncat = i 
   11    continue 
         open(unit=13,file=catname(1:ncat)//'/'//levfil(ilev),
     1			status='old')
         write(line,2011) levfil(ilev)
	 call LogWrite(line,more)
      end if
 2001 format(2x,' No match to list of molecules or isotopomers: ',a)
 2011 format(' Opening level information file: ',a)
c-----------------------------------------------------------------------
c  Read off header junk until you get to START
c-----------------------------------------------------------------------
      write(line,'('' Header from radiative level file follows:'')')
      call LogWrite(line,more)
      do 15 i=1,20
         read(13,'(a)') line
         if(line(1:5) .eq. 'START') go to 16
         call LogWrite(line,more)
   15 continue
   16 continue
c-----------------------------------------------------------------------
c  Read in level energies, quantum numbers, and dipole moments...
c  Levels which are higher than either the max J,K level in the
c  collision rates file or greater than the max J,K selected by the
c  user must be thrown out at this point.
c-----------------------------------------------------------------------
      read(13,'(i2,1x,i2)') nelevs, nflavors
      if(nelevs .eq. MAXLEV) then
	 write(line,'('' Too many radiative levels '')')
	 call LogWrite(line,more)
     	 call bug('f','  MAXLEV must be increased ')
      endif
      ii=1
      do 20 i=1,nelevs
         read(13,1000) j1(ii),km1(ii),kp1(ii),(energy(ii,j),
     1				j=1,nflavors)
         idk = -1
	 do 18 j=1,kmax
	    if(km1(ii) .eq. kval(j)) idk = j
   18	 continue
	 if(idk .gt. 0 .and. j1(ii) .le. jmax(idk)) then
	    jklev(ii) = 10000*j1(ii)+100*km1(ii)+kp1(ii)
            degen(ii) = float(2*j1(ii)+1)
	    ii        = ii + 1
         endif
   20 continue
 1000 format(3(i2,1x),4(3x,e12.6))
      read(13,'(3f5.2)') mua,mub,muc
c-----------------------------------------------------------------------
c  End of reading in energy level information.  Now zero the line
c  strength and Einstein coefficient arrays before reading...
c-----------------------------------------------------------------------
      do 49 i=1,nlev
      do 49 j=1,nlev
         sij(i,j) = 0.0
         a(i,j) = 0.0
         ba(i,j) = 0.0
         be(i,j) = 0.0
	 bbk(i,j) = 0.0
   49 continue
c-----------------------------------------------------------------------
c  Assign the variable MU to the non-zero dipole moment, read the number
c  designations for the upper and lower levels and the line strength for
c  the transition corresponding to these levels, then calculate MU**2
c  times the line strength.
c-----------------------------------------------------------------------
      if(mua.ne.0.0) mu=mua
      if(mub.ne.0.0) mu=mub
      if(muc.ne.0.0) mu=muc
      do 50 im = 1,1000
         read(13,1100) ju,ku,k1u,jl,kl,k1l,ss
         if(ju .eq. 0) go to 55
	 idlo = -1
	 idup = -1
	 jup = 10000*ju+100*ku+k1u
	 jlo = 10000*jl+100*kl+k1l
	 do 45 i=1,nlev
	    if(jklev(i) .eq. jup) idup = i
	    if(jklev(i) .eq. jlo) idlo = i
 45	 continue
	 if(idlo .gt. 0 .and. idup .gt. 0) then
            sij(idup,idlo) = (mu**2)*ss
	 else
	    write(line,'('' No levels '',3(i2,1x),
     1			'' or '',3(i2,1x))') ju,ku,k1u,jl,kl,k1l
	    if(iprint .ge. 1) call LogWrite(line,more)
         endif
 50   continue
 55   continue
 1100    format(3(i2,1x),3(1x,i2),2x,f7.4)
c-----------------------------------------------------------------------
c  Grab the appropriate energies from ENERGY array for given ISOTOP...
c-----------------------------------------------------------------------
      if((isotop.eq.molist(1)).or.(isotop.eq.molist(4))) ident=1
      if((isotop.eq.molist(2)).or.(isotop.eq.molist(5))) ident=2
      if((isotop.eq.molist(3)).or.(isotop.eq.molist(6))) ident=3
c--------------------------------------------------------------------
c  Write out energy levels and their quantum numbers and degeneracies.
c-----------------------------------------------------------------------
      call LogWrite(' ',more)
      call LogWrite('               Energy Levels',more)
      call LogWrite(' ',more)
      write(line,2010)
      call LogWrite(line,more)
 2010 format(2x,'  n   J K-1 K1',5x,1hg,6x,9hw(inv-cm),
     1   2x,6hw(deg))
      call LogWrite(' -----------------------------------------------',
     1			more)
      do 100 i=1,nlev
         ener(i) = energy(i,ident)*29.97925
         x = ener(i)/29.97925
         y = x*1.44
         write(line,2020) i,j1(i),km1(i),kp1(i),degen(i),x,y
	 call LogWrite(line,more)
 2020 format(3x,i2,2x,i2,1x,i2,1x,i2,3x,f5.1,4x,f7.2,3x,f7.2)
  100 continue
c--------------------------------------------------------------------
c  Calculate the Einstein coefficients and the frequencies...
c
c    freq(i,j) = Frequency of transition in GHz
c    a(j,i), ba(i,j), be(j,i) are the Einstein coefficients
c--------------------------------------------------------------------
      f1=2.0e+18*ah/(ac*ac)
      do 300 i=1,nlev
      do 300 j=1,nlev
         if(sij(j,i).eq.0.0) go to 300
         freq(i,j) = ener(j)-ener(i)
         freq(j,i) = freq(i,j)
c-----------------------------------------------------------------------
c  The following IF statement will help the program deal with very
c  low frequency transitions (like the K=3 and 4 K-doublets)...
c-----------------------------------------------------------------------
         if(freq(j,i).le.0.05) go to 300
c
         a(j,i) = 1.162e-11*(freq(j,i)**3)*(sij(j,i)/degen(j))
         be(j,i) = a(j,i)/(f1*(freq(j,i)**3))
         ba(i,j) = be(j,i)*(degen(j)/degen(i))
	 bbk(j,i) = f1*(freq(j,i)**3)/(exp((ah*freq(j,i))/
     1		(ak*tbk))-1.0)
	 if(bbk(j,i) .lt. 1.0e-36) bbk(j,i) = 0.0
  300 continue
      if (iprint.lt.0) go to 900
      call LogWrite(' ',more)
      call LogWrite('                       Molecular Transitions',
     1 			more)
      call LogWrite('    initial    final',more)
      write(line,2040)
 2040 format(3x,8hJ,K1,K-1,2x,8hJ,K1,K-1,4x,9hfreq(GHz),
     2  7x,1hA,11x,2hBA,10x,2hBE)
      call LogWrite(line,more)
      call LogWrite(' ------------------------------------------------',
     1			more)
      do 350 itr=1,nlev
      do 350 jtr=1,nlev
         if(a(itr,jtr).le.0.0) go to 350
         write(line,2050) j1(itr),km1(itr),kp1(itr),j1(jtr),km1(jtr),
     1       kp1(jtr),freq(itr,jtr),a(itr,jtr),ba(jtr,itr),be(itr,jtr)
	 call LogWrite(line,more)
  350 continue
 2050 format(1x,i2,1x,i2,1x,i2,2x,i2,1x,i2,1x,i2,4x,0pf7.2,4x,
     1  3(1pe9.3,3x))
  900 continue
      return
      end
c********************************************************************
c************************** END OF RADASY ***************************
c********************************************************************
