      SUBROUTINE RADLIN(IPRINT)
c-----------------------------------------------------------------------
C      Calculates the Einstein values and frequencies 
C      for a linear molecule. 
C        A  - EINSTEIN SPON. EMIS. COEF. 
C        BE - EINSTEIN STIM EMIS COEF 
C        BA - EINSTEIN ABS COEF 
C------------------------------------------------------------- 
      INTEGER IPRINT

      INCLUDE 'rad_size.h'
      INCLUDE 'radiat.h'
      CHARACTER line*80
      REAL FAC1,FAC2,X
      INTEGER I,J
      LOGICAL MORE
      DATA MORE /.true. /

      FAC1=PI*PI*DY*1.0e-9/AH 
      FAC2=2.0E+18*AH/(AC*AC) 
      DO 20 I=1,NLEV
      DO 20 J=1,NLEV
         A(I,J)   = 0.0
         BE(I,J)  = 0.0
	 BBK(I,J) = 0.0
   20    BA(I,J)  = 0.0
      DO 100 I=1,NLEV 
         X        = I-1.0 
         J        = I-1 
	 JKLEV(I) = 10000*J
         DEGEN(I) = 2.0*X+1.0
         ENER(I)  = BROT*X*(X+1.0)-DE*X*X*(X+1.0)*(X+1.0)
         IF(I.EQ.NLEV) GO TO 100
         FREQ(I+1,I) = 2.0*BROT*(X+1.0)-4.0*(DE*(X+1.0)**3) 
         BE(I+1,I)   = 32.0*FAC1*FAC1/(3.0*AC) 
     1                         *(X+1.0)/(2.*X+3.0) 
         A(I+1,I)    = FAC2*(FREQ(I+1,I)**3)*BE(I+1,I) 
         BA(I,I+1)   = BE(I+1,I)*(2.*X+3.)/(2.*X+1.) 
	 BBK(I+1,I)  = FAC2*(FREQ(I+1,I)**3)/(EXP(AH*FREQ(I+1,I)/
     1                    (AK*TBK))- 1.0)
  100 CONTINUE 
c-----------------------------------------------------------------------
c  Write out energy levels and their quantum numbers and degeneracies.
c    x = Energy in inverse centimeters
c    y = Energy in degrees Kelvin
c-----------------------------------------------------------------------
      call LogWrite(' ',more)
      call LogWrite(' ',more)
      call LogWrite('                  Energy Levels',more)
      call LogWrite(' ',more)
      write(line,2010)
      call LogWrite(line,more)
 2010 format(2x,'  n   J      g       w(inv-cm)   w(Kelvins)')
      call LogWrite(' -----------------------------------------------',
     1                     more)
      do 30 i=1,nlev
         x = ener(i)/29.97925
         y = x*1.44
         write(line,2020) i,i-1,degen(i),x,y
         call LogWrite(line,more)
 30   continue
 2020 format(3x,i2,2x,i2,4x,f5.1,5x,f7.2,5x,f7.2)
c-----------------------------------------------------------------------
c  Write out transition information
c-----------------------------------------------------------------------
      call LogWrite(' ',more)
      call LogWrite(' ',more)
      call LogWrite('                       Molecular Transitions',
     1		more)
      call LogWrite(' ',more)
      WRITE(line,2000)
      CALL LogWrite(line,MORE)
      do 50 i=1,nlev-1
         j = i-1
         WRITE(line,2001) I,J,FREQ(I+1,I),BE(I+1,I),A(I+1,I),
     1                    BA(I,I+1)
         CALL Logwrite(line,MORE)
   50 continue
 2000 FORMAT(4X,2HJI,2X,2HJF,6X,4HFREQ,10X,2HBE,12X, 
     1     2HA ,10X,2HBA) 
 2001 FORMAT(4X,I2,2X,I2,3X,f10.3,3(3X,1PE10.3)) 
      RETURN 
      END
C*************************************************************** 
C*************End of RADLIN************************************* 
C*************************************************************** 
