      subroutine radsym(iprint,isotop)
c-----------------------------------------------------------------------
c    Computes radiative rates for symmetric tops.
c        A  - Einstein spon. emis. coef.
c        BE - Einstein stim emis coef
c        BA - Einstein abs coef
c        B0, C0, DJ, DJK, DK are the rotation constants
c        S - line strength
c-----------------------------------------------------------------------
      INCLUDE 'rad_size.h'
      INCLUDE 'radiat.h'
      real s(MAXLEV,MAXLEV)
      integer i,j,k,nstart,nstop,jlev(MAXLEV),klev(MAXLEV)
      character isotop*10,line*80
      logical more
      data more / .true. /
c-----------------------------------------------------------------------
c  Zero S, A, BA, and BE arrays.
c-----------------------------------------------------------------------
      do 5 i=1,nlev
      do 5 j=1,nlev
         s(i,j) = 0.0
         a(i,j) = 0.0
         ba(i,j) = 0.0
         be(i,j) = 0.0
 5    continue
c-----------------------------------------------------------------------
c  Calculate degeneracies, state energies, frequencies, line strengths,
c  and Einstein coefficients.
c  Note that do-loop goes from J=K (or J lower for the first allowed
c  transition) to the highest J level in each K ladder.
c-----------------------------------------------------------------------
      njk = 0
      do 10 k=1,kmax
         nstart = kval(k)
         nstop = jmax(k)+kval(k)-1
         do 20 j=nstart,nstop
            njk = njk + 1
c-----------------------------------------------------------------------
c  Calculate degeneracies.  Note that gI is different for ortho and para
c  species, and since we are modelling each separately, there is no need
c  to correct for gI.  There is, though, the spin and K degeneracies to
c  take into account.  Note that...
c
c           gJ = 2J + 1
c           gK = 1 for K = 0    (assuming that the inversion levels are
c              = 2 for K not 0   NOT separated, i.e. degenerate)
c-----------------------------------------------------------------------
            degen(njk) = float(2*j+1)
            if(kval(k).ne.0) degen(njk) = 2.0*degen(njk)
c-----------------------------------------------------------------------
c  Now calculate the level energies.  The equation used in the
c  energy calculation is TS (3-54) and that units are GHz.
c-----------------------------------------------------------------------
            ener(njk) = b0*float(j*(j+1)) +
     1                  (c0-b0)*float(kval(k)**2) -
     2                  dj*float(j**2*(j+1)**2) -
     3                  djk*float(j*(j+1)*kval(k)**2) -
     4                  dk*float(kval(k)**4)
            jlev(njk) = j
            klev(njk) = kval(k)
            jklev(njk) = 10000*j+100*kval(k)
 20      continue
 10   continue
c-----------------------------------------------------------------------
c  Write out energy levels and their quantum numbers and degeneracies.
c    x = Energy in inverse centimeters
c    y = Energy in degrees Kelvin
c-----------------------------------------------------------------------
      call LogWrite(' ',more)
      call LogWrite(' ',more)
      call LogWrite('                    Energy Levels',more)
      call LogWrite(' ',more)
      write(line,2010)
      call LogWrite(line,more)
 2010 format(2x,10h  n   J  K,5x,1hg,6x,9hw(inv-cm),
     1   2x,6hw(deg))
      call LogWrite(' -----------------------------------------------',
     1                  more)
      do 30 i=1,nlev
         x = ener(i)/29.97925
         y = x*1.44
         write(line,2020) i,jlev(i),klev(i),degen(i),x,y
         call LogWrite(line,more)
 30   continue
 2020 format(3x,i2,2x,i2,1x,i2,3x,f5.1,4x,f7.2,3x,f7.2)
c-----------------------------------------------------------------------
c  Now calculate line strengths and frequencies.  The equation used
c  to calculate the line strength is TS (3-42) (without the factor
c  of 1/(2*J+1)).
c-----------------------------------------------------------------------
      f1 = 2.0e+18*ah/(ac*ac)
      call LogWrite(' ',more)
      call LogWrite(' ',more)
      call LogWrite('                        Molecular Transitions',
     1                          more)
      call LogWrite('  initial  final',more)
      write(line,2040)
      call LogWrite(line,more)
 2040 format(3x,3hJ,K,4x,3hJ,K,6x,9hfreq(GHz),
     2  7x,1hA,11x,2hBA,10x,2hBE)
      call LogWrite(' -----------------------------------------------',
     1                  more)
c
      njk = 0
      do 40 k=1,kmax
         nstart = kval(k)+1
         nstop = jmax(k)+kval(k)
         do 50 j=nstart,nstop
            njk = njk + 1
            nu = njk + 1
            nl = njk
            if(nu.gt.nlev) go to 50
            if(klev(nu).ne.klev(nl)) then
               s(nu,nl) = 0.0
               freq(nu,nl) = 0.0
               freq(nl,nu) = 0.0
               a(nu,nl) = 0.0
               ba(nl,nu) = 0.0
               be(nu,nl) = 0.0
            else
               s(nu,nl) = float(j**2-kval(k)**2)/float(j)
               freq(nu,nl) = ener(nu) - ener(nl)
               freq(nl,nu) = freq(nu,nl)
               a(nu,nl) = 1.162e-11*freq(nu,nl)**3*
     1               s(nu,nl)*dy**2/(2*j+1)
               be(nu,nl) = a(nu,nl)/(f1*freq(nu,nl)**3)
               ba(nl,nu) = be(nu,nl)*degen(nu)/degen(nl)
            endif
 50      continue
 40   continue
      njk = 0
      do 60 k=1,nlev
         njk = njk + 1
         nu = njk + 1
         nl = njk
         if(a(nu,nl).eq.0.0) go to 60
         write(line,2050) jlev(nu),klev(nu),jlev(nl),
     1         klev(nl),freq(nu,nl),a(nu,nl),ba(nl,nu),
     2         be(nu,nl)
         call LogWrite(line,more)
  60  continue
 2050 format(1x,i2,1x,i2,2x,i2,1x,i2,4x,0pf7.2,4x,3(1pe9.3,3x))
      return
      end
c********************************************************************
c************************** END OF RADSYM ***************************
c********************************************************************
