/*
 *  test out a bunch of things the C version of mirlib does
 *
 *  gcc -g -Imirlib -o testmirlib testmirlib.c mirlib/libmir.a -lm
 * or:
 *  gcc -g -I$MIRINC -I$MIRSUBS -o testmirlib testmirlib.c -L$MIRLIB -lmir -lm
 *  [except it seems to need pgplot now, and it should not !!! ]
 *  gcc -g -I$MIRINC -I$MIRSUBS -o testmirlib testmirlib.c $MIRLIB/libmir.a -lm
 *  [prevents the  pgplot problem]
 *
 *  Note: for LFS you also need to add extra compile flags:
 *     set lfs=(-D_FILE_OFFSET_BITS=64 -D_LARGEFILE_SOURCE -D_LARGEFILE64_SOURCE)
 *     gcc -g $lfs -I$MIRINC -I$MIRSUBS -o testmirlib testmirlib.c $MIRLIB/libmir.a -lm

          +0 +1 +2 +3 +4 +5 +6 +7 +8 +9 +A +B +C +D +E +F    0123456789ABCDEF

 +00000000 61 5F 69 74 65 6D 00 00 00 00 00 00 00 00 00 00    a_item..........
  00000010 61 5F 63 68 61 72 00 00 00 00 00 00 00 00 00 09    a_char..........
  00000020 00 00 00 01 48 65 6C 6C 6F 00 00 00 00 00 00 00    ....Hello.......
  00000030 61 5F 69 6E 74 38 00 00 00 00 00 00 00 00 00 08    a_int8..........
  00000040 00 00 00 02 00 00 00 1C 00 00 00 00 00 00 00 00    ................
  00000050 61 5F 69 6E 74 00 00 00 00 00 00 00 00 00 00 08    a_int...........
  00000060 00 00 00 02 00 00 00 1C 00 00 00 00 00 00 00 00    ................
  00000070 61 5F 72 65 61 6C 5F 64 00 00 00 00 00 00 00 08    a_real_d........
  00000080 00 00 00 04 40 49 0F DB 00 00 00 00 00 00 00 00    ....@I..........
  00000090 61 5F 72 65 61 6C 5F 66 00 00 00 00 00 00 00 08    a_real_f........
  000000A0 00 00 00 04 40 49 0F DB 00 00 00 00 00 00 00 00    ....@I..........
  000000B0 61 5F 64 6F 75 62 6C 65 00 00 00 00 00 00 00 10    a_double........
  000000C0 00 00 00 05 58 01 C0 00 40 09 21 FB 54 44 2D 18    ....X...@.!.TD-.

 *
 * History:
 *
 *      
 *      aug-2006:     some changes for MIR5
 *      oct-2008      append option, added more to 's' to show things
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>

#include "maxdimc.h"
#include "miriad.h"


#define check(iostat) if(iostat)bugno_c('f',iostat)


void test_hio(char *name1)
{
  int t1, i1,iostat;
  double pi_d = 3.14159265358979323846;
  float  pi_f = 3.14159265358979323846;
  int  d  = 258;          /* 0x01 0x02 */
  int8 d8_0 = 258;
  int8 d8_1 = 2147483647LL; /*      0x0F 0xFF 0xFF 0xFF */
  int8 d8_2 = 2147483648LL; /*      0x0F 0xFF 0xFF 0xFF */
  int8 d8_3 = 4294967295LL; /* 0x01 0x00 0x00 0x00 0x02 */
  int8 d8_4 = 4294967296LL; /* 0x01 0x00 0x00 0x00 0x02 */
  int8 d8_5 = 4294967298LL; /* 0x01 0x00 0x00 0x00 0x02 */

  fprintf(stderr,"test_hio: %s\n",name1);

  hopen_c(&t1, name1, "old", &iostat);
  if (iostat==0) {
    fprintf(stderr,"Deleting previous dataset %s\n",name1);
    hrm_c(t1);
  } else
    fprintf(stderr,"Creating new dataset %s\n",name1);

  hopen_c(&t1, name1, "new", &iostat);                check(iostat);

  wrhdd_c(t1,"a_double",pi_d);
  wrhdr_c(t1,"a_real_f",pi_f);
  wrhdr_c(t1,"a_real_d",pi_d);
  wrhdi_c(t1,"a_int",d);
  wrhdl_c(t1,"a_int8_0",d8_0);
  wrhdl_c(t1,"a_int8_1",d8_1);
  wrhdl_c(t1,"a_int8_2",d8_2);
  wrhdl_c(t1,"a_int8_3",d8_3);
  wrhdl_c(t1,"a_int8_4",d8_4);
  wrhdl_c(t1,"a_int8_5",d8_5);
  wrhda_c(t1,"a_char","Hello");
  wrhda_c(t1,"a_char_1","Hello World");

  haccess_c(t1,&i1,"a_item","write",&iostat);         check(iostat);
  hdaccess_c(i1,&iostat);                             check(iostat);

#if 0
  hdelete_c(t1,"a_char",&iostat);                          check(iostat);
#endif
  hclose_c(t1);

}

void test_xyio(char *fname, int nx, int ny, int nz)
{
  int t1, i,j,k,iostat;
  float data[MAXDIM];
  int axes[3];

  fprintf(stderr,"test_xyio: %s nx,ny,nz=%d %d %d\n",fname,nx,ny,nz);

  /* delete old one , if exists */
  hopen_c(&t1, fname, "old", &iostat);
  if (iostat==0) hrm_c(t1);

  axes[0] = nx;
  axes[1] = ny;
  axes[2] = nz;

  xyopen_c(&t1,fname,"new",3,axes);
  for (k=1; k<=nz; k++) {
    xysetpl_c(t1,1,&k);
    for (j=1; j<=ny; j++)
      xywrite_c(t1,j,data);
  }
  xyclose_c(t1);


}

void test_uvio(char *fname, int nc, int nw, int nr)
{
  int t1, i,iostat;
  double preamble[5];
  float data[2*MAXCHAN];
  int  flags[MAXCHAN];
  int nschan  = 1;

  fprintf(stderr,"test_uvio: %s nc,nw,nr=%d %d %d\n",fname,nc,nw,nr);

  /* check if old one , if exists, add data in append mode */
  hopen_c(&t1, fname, "old", &iostat);

  if (iostat==0)  {
    fprintf(stderr,"  appending to %s\n",fname);
    uvopen_c(&t1, fname, "append");
  } else {
    fprintf(stderr,"  new dataset %s\n",fname);
    uvopen_c(&t1, fname, "new");
  }

  uvset_c(t1, "data",     "channel", 0, 1.0, 1.0, 1.0);
  uvset_c(t1, "preamble", "uvw/time/baseline", 0, 0.0, 0.0, 0.0);

  for (i=0; i<nr; i++) {
    /* garbage uv variables */
    uvputvr_c(t1,H_BYTE,"abyte",(char *)data, 1);
    uvputvr_c(t1,H_INT2,"aint2",(char *)data, 1);
    uvputvr_c(t1,H_INT, "aint4",(char *)data, 1);
    uvputvr_c(t1,H_REAL,"areal",(char *)data, 1);
    uvputvr_c(t1,H_DBLE,"adble",(char *)data, 1);
    uvputvr_c(t1,H_CMPLX,"acmplx",(char *)data, 1);
    /* some data */
    if (nw)  uvwwrite_c(t1,data,flags,nw);
    uvwrite_c(t1,preamble,data,flags,nc);
  }
  uvclose_c(t1);
}


void test_sizes(void)
{
  printf("sizeof(short)     = %d\n",sizeof(short));
  printf("sizeof(int)       = %d\n",sizeof(int));
  printf("sizeof(long)      = %d\n",sizeof(long));
  printf("sizeof(long long) = %d\n",sizeof(long long));
  printf("sizeof(size_t)    = %d\n",sizeof(size_t));
  printf("sizeof(off_t)     = %d\n",sizeof(off_t));
  printf("sizeof(void *)    = %d\n",sizeof(void *));

  printf("MIRTEL            = %s\n",MIRTEL);
  printf("MAXIANT           = %d\n",MAXIANT);
  printf("MAXANT            = %d\n",MAXANT);
  printf("MAXBASE           = %d\n",MAXBASE);
  printf("MAXDIM            = %d\n",MAXDIM);
  printf("MAXCHAN           = %d\n",MAXCHAN);
  printf("MAXWIN            = %d\n",MAXWIN);
  
}

int main(int argc, char *argv[])
{
  int n1, n2, n3;
  char *buf;
  fprintf(stderr,"Testing MIRLIB:\n");
  if (argc==1) {
    fprintf(stderr,"Command line options\n");
    fprintf(stderr," h                              hio test on test1.mir \n");
    fprintf(stderr," x [nx ny nz]                   xyio write test on test1.xy\n");
    fprintf(stderr," u [nc nw nr]                   uvio write test on test1.uv\n");
    fprintf(stderr," m                              malloc loop until full memory [sic]\n");
    fprintf(stderr," s                              show sizeof() and MAX... of things\n");
    return 1;
  }

  switch (*argv[1]) {
  case 'h': 
    test_hio("test1.mir");
    break;
  case 'x':
    if (argc>2) {
      n1 = atoi(argv[2]);
      n2 = atoi(argv[3]);
      n3 = atoi(argv[4]);
    } else {
      n1 = n2 = n3 = 64;
    }
    test_xyio("test1.xy", n1,n2,n3);
    break;
  case 'u':
    if (argc>2) {
      n1 = atoi(argv[2]);
      n2 = atoi(argv[3]);
      n3 = atoi(argv[4]);
    } else {
      n1 = 1024;
      n2 = 16;
      n3 = 40000;
    }
    test_uvio("test1.uv", n1,n2,n3);
    break;
  case 'm':
    fprintf(stderr,"Malloc loop until memory full, incrementing by %d ... ",BUFSIZE);  
    sleep(1);
    fprintf(stderr,"... go!\n");
    do {
      buf = malloc(BUFSIZE);
    } while (buf);
    break;
  case 's':
    test_sizes();
    break;
  default:
    break;
  }
  return 0;
}
